!---------------------------------------------------------------------------------------
! PREP_FLEXPART: read_wdcgg
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> read_wdcgg
!!
!! Purpose:    Reads observations from WDCGG files and averages and/or filters them.
!!             Writes the averaged/filtered observations to file in the format 
!!             required by FLEXINVERT.
!!
!! Interface:
!!
!!    Inputs
!!             settings  -  settings data structure
!!             jd        -  julian day for start of month
!!             nr        -  index to the receptors list
!!
!!    Outputs
!!             nobs      -  number of observations
!!             obs       -  observations data structure
!!
!!    Externals
!!             caldate
!!             alloc_obs
!!             parse_string
!!             sort
!!
!---------------------------------------------------------------------------------------

subroutine read_wdcgg(settings, jd, nr, nobs, obs)

  use mod_settings
  use mod_var
  use mod_dates
  use mod_obs
  use mod_strings
  use mod_tools

  implicit none

  type (settings_t), intent(in)                         :: settings
  real(kind=8),      intent(in)                         :: jd
  integer,           intent(in)                         :: nr
  integer,           intent(out)                        :: nobs
  type (obs_t),      intent(out)                        :: obs

  character(len=max_path_len)                           :: file_out
  character(len=max_path_len)                           :: file_obs
  character(len=200)                                    :: line
  character(len=200), dimension(3)                      :: args
  character(len=10)                                     :: adate, char10
  character(len=5)                                      :: atime, flag, char5
  character(len=4)                                      :: ayear
  logical                                               :: lexist
  integer                                               :: ierr
  integer                                               :: cnt 
  integer                                               :: narg, nflag
  integer                                               :: cs, rem
  integer                                               :: c, i, j, m, n, p, q
  integer                                               :: jjjjmmdd, hhmiss, yyyy, mm, dd, hh, mi, ss
  integer                                               :: eomday, hloc, num
  real                                                  :: conc, sd, err, lon, lat, alt, freq
  real(kind=8)                                          :: jdate, jdateutc
  real(kind=8), dimension(1000)                         :: jdobs  
  real, dimension(1000)                                 :: latobs, lonobs, altobs, concobs, errobs

  call caldate(jd, jjjjmmdd, hhmiss)
  eomday = calceomday(jjjjmmdd/100)
  write(ayear,fmt='(I4.4)') jjjjmmdd/10000

  freq = 1.

  do n = 1, nfiles
    do i = 1, len_trim(filelist(n))-2
      if ( filelist(n)(i:(i+2)).eq.to_lower(trim(recname(nr))) ) then
        do j = 1, len_trim(filelist(n))-3 
          if ( filelist(n)(j:(j+3)).eq.ayear ) go to 10
        end do
      endif
    end do
  end do
10 continue
  file_obs = filelist(n)

  ! open obs file
  open(100,file=trim(settings%path_obs)//trim(file_obs),action='read',status='old',iostat=ierr)
  if ( ierr.ne.0 ) then
    print*, 'ERROR: cannot open: '//trim(settings%path_obs)//trim(file_obs)
    stop
  endif 
  print*, 'Reading file: '//trim(settings%path_obs)//trim(file_obs)

  ! read header
  n = len_trim('LATITUDE:')
  m = len_trim('LONGITUDE:')
  p = len_trim('ALTITUDE:')
  q = len_trim('DATE  TIME')
  do while (ierr.eq.0)
    read (100, fmt='(A)', iostat=ierr) line
    if ( line(7:(q+6)) == 'DATE  TIME' ) ierr = 1
    if ( line(5:(n+4)) == 'LATITUDE:' ) then
      call parse_string(line, ":", args(:), narg)
      read(args(2),*) lat
    endif
    if ( line(5:(m+4)) == 'LONGITUDE:' ) then
      call parse_string(line, ":", args(:), narg)
      read(args(2),*) lon
    endif
    if ( line(5:(p+4)) == 'ALTITUDE:' ) then
      call parse_string(line, ":", args(:), narg)
      read(args(2),*) alt
    endif
  end do

  print*, 'lat, lon, alt: ',lat, lon, alt

  ! read data
  ! note: local time stamp
  cnt = 0
  read_loop: do 
    read(100,fmt='(A10,1X,A5,1X,A10,1X,A5,1X,F10.2,1X,I5,1X,F8.2,1X,A5,1X,I1,1X,I8)',iostat=ierr) &
             adate, atime, char10, char5, conc, num, sd, flag, cs, rem
    if(ierr.ne.0) exit read_loop
    call parse_string(adate, "-", args(:), narg)
    read(args(1:3),*) yyyy, mm, dd
    call parse_string(atime, ":", args(:), narg)
    read(args(1:2),*) hh, mi
    read(flag,*) nflag
    ! calculate julian date 
    jjjjmmdd = yyyy*10000+mm*100+dd
    hhmiss = hh*10000+mi*100
    jdate = juldate(jjjjmmdd, hhmiss)
    ! adjust to UTC
    jdateutc = jdate - dint(real(lon,kind=8)*24d0/360d0)/24d0
    if ( conc.le.-999. ) cycle read_loop
    if ( nflag.ne.0. ) cycle read_loop
    if ( (jdateutc.lt.jd).or.(jdate.ge.min(jreldatef,(jd+real(eomday)))) ) cycle read_loop
    cnt = cnt + 1
    jdobs(cnt) = jdateutc
    latobs(cnt) = lat
    lonobs(cnt) = lon
    altobs(cnt) = alt
    concobs(cnt) = conc
    errobs(cnt) = settings%measerr
  end do read_loop
  nobs = cnt

  ! close input file
  close(100)

  print*, 'nobs: ',nobs

  ! no observations go to end
  if ( nobs.eq.0 ) then
    print*, 'WARNING: no data for '//trim(recname(nr))
    go to 20
  endif

  ! testing
!  do i = 1, nobs
!    print*, jdobs(i),latobs(i),lonobs(i),altobs(i),concobs(i),errobs(i)
!  end do

  ! allocate obs
  call alloc_obs(nobs, obs)

  print*, 'jdobs(nobs):',jdobs(nobs)

  ! average and/or select observations
  call process_obs(settings, nobs, freq, jdobs, latobs, lonobs, altobs, concobs, errobs, obs)

  ! no observations after selection
  if ( nobs.eq.0 ) go to 20

  ! write formatted obs to file
  file_out = trim(settings%path_obsout)//trim(recname(nr))//'_'//trim(settings%species)//'.txt'
  inquire(file=trim(file_out),exist=lexist)
  if( lexist ) then
    ! append to existing
    print*, 'WARNING: appending to file '//trim(file_out)
    open(100,file=trim(file_out),status='old',action='write',access='append',iostat=ierr)
    do i = 1, nobs
      call caldate(obs%jdi(i), jjjjmmdd, hhmiss)
      write(100,fmt='(A3,1X,I8,1X,I6,1X,F14.6,1X,F10.4,1X,F10.4,1X,I4)') &
              trim(recname(nr)), jjjjmmdd, hhmiss, obs%jdi(i), obs%con(i), obs%err(i), obs%num(i)
    end do
    close(100)
  else
    ! create new file
    open(100,file=trim(file_out),status='new',action='write',iostat=ierr)
    write(100,fmt='(A)') 'rec yyyymmdd hhmmss juldate conc err num'
    do i = 1, nobs
      call caldate(obs%jdi(i), jjjjmmdd, hhmiss)
      write(100,fmt='(A3,1X,I8,1X,I6,1X,F14.6,1X,F10.4,1X,F10.4,1X,I4)') &
              trim(recname(nr)), jjjjmmdd, hhmiss, obs%jdi(i), obs%con(i), obs%err(i), obs%num(i)
    end do
    close(100)
  endif

  ! write detailed station list file
  file_out = trim(settings%path_obsout)//'stnlist_detail.txt'
  inquire(file=trim(file_out),exist=lexist)
  if( lexist ) then
    ! append to existing
    open(100,file=trim(file_out),status='old',action='write',access='append',iostat=ierr)
    write(100,fmt='(A3,1X,F6.2,1X,F6.2,1X,F6.1)') trim(recname(nr)), obs%lat(1), obs%lon(1), obs%alt(1)
    close(100)
  else
    ! create new file
    open(100,file=trim(file_out),status='new',action='write',iostat=ierr)
    write(100,fmt='(A3,1X,F6.2,1X,F6.2,1X,F6.1)') trim(recname(nr)), obs%lat(1), obs%lon(1), obs%alt(1)
    close(100)
  endif

20 continue

end subroutine read_wdcgg


