!---------------------------------------------------------------------------------------
! FLEXINVERT : correl
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> correl 
!! 
!! Purpose:    Calculates spatial correlation between grid cells using exponential 
!!             decay with distance. Used in the calculation of the error covariance 
!!             matrix.
!!
!! Interface:
!!
!!    Inputs
!!             config  -  configuration settings data structure
!!             corr    -  correlation matrix (zero matrix on input)
!!
!---------------------------------------------------------------------------------------

subroutine correl(config, corr)

  use mod_var
  use mod_settings

  implicit none

  type (config_t),            intent (in)    :: config
  real, dimension(nbox,nbox), intent(in out) :: corr

  integer                                    :: ix, jy 
  integer                                    :: ind1, ind2
  real                                       :: indx, indy
  real                                       :: pih, cor, dd, ddx, num
  real                                       :: lat1, lat2, lon1, lon2
  real                                       :: sigma1, sigma2


  pih = pi/180.

  do ind1 = 1, nbox
    indx = 0.
    indy = 0.
    num = 0.
    do jy = 1, nyregrid
      do ix = 1, nxregrid
        if ( nbox_xy(ix,jy).eq.ind1 ) then
          if ( lsm_box(ix,jy).eq.1 ) then
            sigma1 = config%sigma_land
          else
            sigma1 = config%sigma_ocean
          endif
          indx = indx + real(ix)
          indy = indy + real(jy)
          num = num + 1.
        end if
      end do
    end do
    indx = anint(indx/num)
    indy = anint(indy/num)
    lon1 = rllx+(indx-1.)*rdx
    lat1 = rlly+(indy-1.)*rdy
    do ind2 = ind1, nbox
      indx = 0.
      indy = 0.
      num = 0.
      do jy = 1, nyregrid
        do ix = 1, nxregrid
          if ( nbox_xy(ix,jy).eq.ind2 ) then
            if ( lsm_box(ix,jy).eq.1 ) then
              sigma2 = config%sigma_land
            else
              sigma2 = config%sigma_ocean
            endif
            indx = indx + real(ix)
            indy = indy + real(jy)
            num = num + 1.
          end if
        end do
      end do
      indx = anint(indx/num)
      indy = anint(indy/num)
      lon2 = rllx + (indx-1.)*rdx
      lat2 = rlly + (indy-1.)*rdy
      if ( ind1.eq.ind2 ) then
        ddx = 0.
      else
        dd = (1. - sin(lat2*pih)*sin(lat1*pih) - &
               cos(lat2*pih)*cos(lat1*pih)*cos((lon2-lon1)*pih))/2.
        dd = max(dd,0.)
        ddx = rearth*2.*asin(sqrt(dd))
      endif
      cor = exp(-1.*ddx/sigma1)
      ! no correlation between land and ocean
      if ( sigma1.ne.sigma2 ) cor = 0.
      if ( cor.lt.0.0001 ) cor = 0.
      ! and to be sure
      if ( ind1.eq.ind2 ) cor = 1.
      corr(ind1,ind2) = cor
      corr(ind2,ind1) = cor
    end do ! ind2
  end do ! ind1


end subroutine correl

